/* 
 * File:   NFTDialog.cpp
 * Author: binnette
 * 
 * Created on 5 juin 2010, 16:10
 */
#include <iostream>
#include "NFTDialog.h"
#include "categories.h"

NFTDialog::NFTDialog() :
//widgets
m_button_ok(Gtk::Stock::OK), //const StockID& stock_id
m_button_cancel(Gtk::Stock::CANCEL),
m_frame_categories("Categories:"), //const Glib::ustring& label
m_frame_templates("Templates:"),
m_frame_preview_info("Preview and info:"),
m_hbox_frame(false, 10), //bool homogeneous=false, int spacing=0
m_hbox_action(false, 10),
m_image_preview(),
m_treeview_categories(),
m_treeview_templates(),
m_vbox_global(false, 10),
m_vbox_preview_info(false, 10),
//storage
m_columns_categories(),
m_columns_templates()
//begin
{
    //Window properties
    set_title("New From Template Dialog");
    set_border_width(10);
    set_size_request(600, 400);

    //Categories storage
    m_refListStore_categories = Gtk::ListStore::create(m_columns_categories);
    m_treeview_categories.set_model(m_refListStore_categories);
    m_treeview_categories.append_column("Name", m_columns_categories.m_col_categorie_name);
    //m_treeview_categories.append_column("Index", m_columns_categories.m_col_categorie_index);
    m_treeview_categories.set_headers_visible(false);

    create_categories();

    //Template storage
    m_refListStore_templates = Gtk::ListStore::create(m_columns_templates);
    m_treeview_templates.set_model(m_refListStore_templates);
    m_treeview_templates.append_column("Name", m_columns_templates.m_col_template_name);
    //m_treeview_templates.append_column("Index", m_columns_templates.m_col_template_index);
    m_treeview_templates.set_headers_visible(false);

    //show templates for category 0
    if (m_categories.size() > 0) {
        show_template_from_categorie_index(0);
    }

    //Image
    m_image_preview.set("image_preview.png");
    m_image_preview.set_size_request(279, 150);

    //Template storage
    m_refListStore_info = Gtk::ListStore::create(m_columns_info);
    m_treeview_info.set_model(m_refListStore_info);
    m_treeview_info.append_column("Tag", m_columns_info.m_col_template_tag);
    m_treeview_info.append_column("Value", m_columns_info.m_col_template_value);

    m_template_selected = NULL;

    //Signal handlers

    //on_button_ok_clicked
    m_button_ok.signal_clicked().connect(
            sigc::mem_fun(*this, &NFTDialog::on_button_ok_clicked));

    //on_button_cancel_clicked
    m_button_cancel.signal_clicked().connect(
            sigc::mem_fun(*this, &NFTDialog::on_button_cancel_clicked));

    //on_treeview_categories_selection_changed
    refTreeSelection_categories = m_treeview_categories.get_selection();
    refTreeSelection_categories->signal_changed().connect(
            sigc::mem_fun(*this, &NFTDialog::on_treeview_categories_selection_changed));

    //on_treeview_templates_selection_changed
    refTreeSelection_templates = m_treeview_templates.get_selection();
    refTreeSelection_templates->signal_changed().connect(
            sigc::mem_fun(*this, &NFTDialog::on_treeview_templates_selection_changed));

    //add components
    add(m_vbox_global);
    m_vbox_global.pack_start(m_hbox_frame, true, true, 0); //Widget& child, bool expand, bool fill, guint padding = 0	 
    //frame categories
    m_hbox_frame.pack_start(m_frame_categories, false, true, 0);
    m_frame_categories.add(m_treeview_categories);
    //frame templates
    m_hbox_frame.pack_start(m_frame_templates, false, true, 0);
    m_frame_templates.add(m_treeview_templates);
    //frame previex and info
    m_hbox_frame.pack_start(m_frame_preview_info, true, true, 0);
    m_frame_preview_info.add(m_vbox_preview_info);
    m_vbox_preview_info.pack_start(m_image_preview, false, true, 0);
    m_vbox_preview_info.pack_start(m_treeview_info, true, true, 0);
    //user action
    m_vbox_global.pack_start(m_hbox_action, false, false, 0);
    m_hbox_action.pack_end(m_button_ok, false, false, 0);
    m_hbox_action.pack_end(m_button_cancel, false, false, 0);

    // The final step is to display this newly created widget...
    this->show_all();
}

NFTDialog::~NFTDialog() {
}

void NFTDialog::on_button_ok_clicked() {
    std::cout << "button_ok_clicked" << std::endl;
    if (m_template_selected != NULL) {
        std::cout << "you have selected template named ";
        std::cout << m_template_selected->get_name();
        std::cout << std::endl;
    }
    std::cout << "doing nothing" << std::endl;
}

void NFTDialog::on_button_cancel_clicked() {
    std::cout << "button_cancel_clicked" << std::endl;
    std::cout << "closing application..." << std::endl;
    Gtk::Main::quit();
}

void NFTDialog::on_treeview_categories_selection_changed() {
    Gtk::TreeModel::iterator iter = refTreeSelection_categories->get_selected();
    if (iter) //If anything is selected
    {
        Gtk::TreeModel::Row row = *iter;
        int index = row[m_columns_categories.m_col_categorie_index];
        show_template_from_categorie_index(index);
    }
}

void NFTDialog::on_treeview_templates_selection_changed() {
    Gtk::TreeModel::iterator iter = refTreeSelection_templates->get_selected();
    if (iter) //If anything is selected
    {
        Gtk::TreeModel::Row row = *iter;
        int index = row[m_columns_templates.m_col_template_index];
        show_info_from_template_index(index);
    }
}

void NFTDialog::show_template_from_categorie_index(int index) {
    Category category = m_categories.at(index);
    m_templates_currents = category.get_vector();
    Gtk::TreeModel::iterator iter;
    Gtk::TreeModel::Row row;

    //clear list
    m_refListStore_templates->clear();

    for (int i = 0; i < m_templates_currents.size(); i++) {
        Template temp = m_templates_currents.at(i);
        iter = m_refListStore_templates->append();
        row = *iter;
        row[m_columns_templates.m_col_template_name] = temp.get_name();
        row[m_columns_categories.m_col_categorie_index] = i;
    }
}

void NFTDialog::show_info_from_template_index(int index) {
    Gtk::TreeModel::iterator iter;
    Gtk::TreeModel::Row row;
    std::cout << "call" << std::endl;
    //clear list
    m_refListStore_info->clear();

    Template t = m_templates_currents.at(index);
    m_template_selected = new Template(t.get_name(), t.get_author(), t.get_licence(), t.get_date());

    //name
    iter = m_refListStore_info->append();
    row = *iter;
    row[m_columns_info.m_col_template_tag] = "Name";
    row[m_columns_info.m_col_template_value] = t.get_name();
    //author
    iter = m_refListStore_info->append();
    row = *iter;
    row[m_columns_info.m_col_template_tag] = "Author";
    row[m_columns_info.m_col_template_value] = t.get_author();
    //licence
    iter = m_refListStore_info->append();
    row = *iter;
    row[m_columns_info.m_col_template_tag] = "Licence";
    row[m_columns_info.m_col_template_value] = t.get_licence();
    //date
    iter = m_refListStore_info->append();
    row = *iter;
    row[m_columns_info.m_col_template_tag] = "Date";
    row[m_columns_info.m_col_template_value] = t.get_date();
}

void NFTDialog::create_categories() {
    //Create and add examples categories
    m_categories.push_back(Category("Covers"));
    m_categories.push_back(Category("Default"));
    m_categories.push_back(Category("Desktops"));
    m_categories.push_back(Category("Fonts"));
    m_categories.push_back(Category("Icons"));
    m_categories.push_back(Category("Labels"));
    m_categories.push_back(Category("Paper sizes"));
    m_categories.push_back(Category("Slideshows"));
    m_categories.push_back(Category("Template themes"));
    m_categories.push_back(Category("Uncategorized"));
    m_categories.push_back(Category("Videos"));
    m_categories.push_back(Category("Web"));

    Category category;
    Gtk::TreeModel::iterator iter;
    Gtk::TreeModel::Row row;

    for (int i = 0; i < m_categories.size(); i++) {
        category = m_categories.at(i);
        iter = m_refListStore_categories->append();
        row = *iter;
        row[m_columns_categories.m_col_categorie_name] = category.get_name();
        row[m_columns_categories.m_col_categorie_index] = i;
    }



}